@@ -1,11 +1,9 @@
 -- CFB.hs: OpenPGP (RFC4880) CFB mode
--- Copyright © 2013-2016  Clint Adams
+-- Copyright © 2013-2018  Clint Adams
 -- Copyright © 2013  Daniel Kahn Gillmor
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE CPP #-}
-
 module Codec.Encryption.OpenPGP.CFB (
    decrypt
  , decryptNoNonce
@@ -16,9 +14,6 @@ module Codec.Encryption.OpenPGP.CFB (
 import Codec.Encryption.OpenPGP.BlockCipher (withSymmetricCipher)
 import Codec.Encryption.OpenPGP.Internal.HOBlockCipher
 import Codec.Encryption.OpenPGP.Types
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>), (<*>))
-#endif
 import qualified Data.ByteString as B
 
 decryptOpenPGPCfb :: SymmetricAlgorithm -> B.ByteString -> B.ByteString -> Either String B.ByteString
