@@ -1,5 +1,5 @@
 -- Internal.hs: private utility functions and such
--- Copyright © 2012-2016  Clint Adams
+-- Copyright © 2012-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -19,6 +19,10 @@ module Codec.Encryption.OpenPGP.Internal (
  , curveoidBSToCurve
  , curveToCurveoidBS
  , point2BS
+ , curveoidBSToEdSigningCurve
+ , edSigningCurveToCurveoidBS
+ , curve2Curve
+ , curveFromCurve
 ) where
 
 import Crypto.Number.Serialize (i2osp, os2ip)
@@ -32,7 +36,6 @@ import qualified Data.ByteString as B
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString.Lazy as BL
 import Data.List (find)
-import Data.Maybe (fromJust)
 import Data.Word (Word8, Word16)
 
 import Codec.Encryption.OpenPGP.Types
@@ -72,8 +75,10 @@ pubkeyToMPIs (DSAPubKey (DSA_PublicKey k)) = [
   where pkParams f = MPI . f . DSA.public_params $ k
 
 pubkeyToMPIs (ElGamalPubKey p g y) = [MPI p, MPI g, MPI y]
-pubkeyToMPIs (ECDHPubKey (ECDSA_PublicKey (ECDSA.PublicKey _ q)) _ _) = [MPI (os2ip (point2BS q))]
-pubkeyToMPIs (ECDSAPubKey (ECDSA_PublicKey (ECDSA.PublicKey _ q))) = [MPI (os2ip (point2BS q))]
+pubkeyToMPIs (ECDHPubKey (ECDSAPubKey (ECDSA_PublicKey (ECDSA.PublicKey _ q))) _ _) = [MPI (os2ip (point2BS q))]
+pubkeyToMPIs (ECDHPubKey (EdDSAPubKey _ (EPoint x)) _ _) = [MPI x]
+pubkeyToMPIs (ECDSAPubKey ((ECDSA_PublicKey (ECDSA.PublicKey _ q)))) = [MPI (os2ip (point2BS q))]
+pubkeyToMPIs (EdDSAPubKey _ (EPoint x)) = [MPI x]
 
 multiplicativeInverse :: Integral a => a -> a -> a
 multiplicativeInverse _ 1 = 1
@@ -100,22 +105,40 @@ sigCT (SigV3 _ ct _ _ _ _ _) = Just ct
 sigCT (SigV4 _ _ _ hsubs _ _ _) = fmap (\(SigSubPacket _ (SigCreationTime i)) -> i) (find isSigCreationTime hsubs)
 sigCT _ = Nothing
 
-curveoidBSToCurve :: B.ByteString -> Either String ECCT.Curve
+curveoidBSToCurve :: B.ByteString -> Either String ECCCurve
 curveoidBSToCurve oidbs
-    | B.pack [0x2A,0x86,0x48,0xCE,0x3D,0x03,0x01,0x07] == oidbs = Right $ ECCT.getCurveByName ECCT.SEC_p256r1
-    | B.pack [0x2B,0x81,0x04,0x00,0x22] == oidbs = Right $ ECCT.getCurveByName ECCT.SEC_p384r1
-    | B.pack [0x2B,0x81,0x04,0x00,0x23] == oidbs = Right $ ECCT.getCurveByName ECCT.SEC_p521r1
-    | otherwise = Left "unknown curve OID"
+    | B.pack [0x2A,0x86,0x48,0xCE,0x3D,0x03,0x01,0x07] == oidbs = Right $ NISTP256 -- ECCT.getCurveByName ECCT.SEC_p256r1
+    | B.pack [0x2B,0x81,0x04,0x00,0x22] == oidbs = Right $ NISTP384 -- ECCT.getCurveByName ECCT.SEC_p384r1
+    | B.pack [0x2B,0x81,0x04,0x00,0x23] == oidbs = Right $ NISTP521 -- ECCT.getCurveByName ECCT.SEC_p521r1
+    | B.pack [0x2B,0x06,0x01,0x04,0x01,0x97,0x55,0x01,0x05,0x01] == oidbs = Right Curve25519
+    | otherwise = Left $ concat ["unknown curve (...", show (B.unpack oidbs) ,")"]
 
--- [0x2B 0x06 0x01 0x04 0x01 0xDA 0x47 0x0F 0x01] -- ed25519
-
-curveToCurveoidBS :: ECCT.Curve -> Either String B.ByteString
-curveToCurveoidBS curve
-    | curve == ECCT.getCurveByName ECCT.SEC_p256r1 = Right $ B.pack [0x2A,0x86,0x48,0xCE,0x3D,0x03,0x01,0x07]
-    | curve == ECCT.getCurveByName ECCT.SEC_p384r1 = Right $ B.pack [0x2B,0x81,0x04,0x00,0x22]
-    | curve == ECCT.getCurveByName ECCT.SEC_p521r1 = Right $ B.pack [0x2B,0x81,0x04,0x00,0x23]
-    | otherwise = Left "unknown curve"
+curveToCurveoidBS :: ECCCurve -> Either String B.ByteString
+curveToCurveoidBS NISTP256 = Right $ B.pack [0x2A,0x86,0x48,0xCE,0x3D,0x03,0x01,0x07]
+curveToCurveoidBS NISTP384 = Right $ B.pack [0x2B,0x81,0x04,0x00,0x22]
+curveToCurveoidBS NISTP521 = Right $ B.pack [0x2B,0x81,0x04,0x00,0x23]
+curveToCurveoidBS Curve25519 = Right $ B.pack [0x2B,0x06,0x01,0x04,0x01,0x97,0x55,0x01,0x05,0x01]
+curveToCurveoidBS _ = Left "unknown curve"
 
 point2BS :: ECCT.PublicPoint -> B.ByteString
 point2BS (ECCT.Point x y) = B.concat [B.singleton 0x04, i2osp x, i2osp y] -- FIXME: check for length equality?
 point2BS ECCT.PointO = error "FIXME: point at infinity"
+
+curveoidBSToEdSigningCurve :: B.ByteString -> Either String EdSigningCurve
+curveoidBSToEdSigningCurve oidbs
+    | B.pack [0x2B,0x06,0x01,0x04,0x01,0xDA,0x47,0x0F,0x01] == oidbs = Right Ed25519
+    | otherwise = Left $ concat ["unknown Edwards signing curve (...", show (B.unpack oidbs) ,")"]
+
+edSigningCurveToCurveoidBS :: EdSigningCurve -> Either String B.ByteString
+edSigningCurveToCurveoidBS Ed25519 = Right $ B.pack [0x2B,0x06,0x01,0x04,0x01,0xDA,0x47,0x0F,0x01]
+
+curve2Curve :: ECCCurve -> ECCT.Curve
+curve2Curve NISTP256 = ECCT.getCurveByName ECCT.SEC_p256r1
+curve2Curve NISTP384 = ECCT.getCurveByName ECCT.SEC_p384r1
+curve2Curve NISTP521 = ECCT.getCurveByName ECCT.SEC_p521r1
+
+curveFromCurve :: ECCT.Curve -> ECCCurve
+curveFromCurve c
+    | c == ECCT.getCurveByName ECCT.SEC_p256r1 = NISTP256
+    | c == ECCT.getCurveByName ECCT.SEC_p384r1 = NISTP384
+    | c == ECCT.getCurveByName ECCT.SEC_p521r1 = NISTP521
