@@ -1,9 +1,8 @@
 -- KeySelection.hs: OpenPGP (RFC4880) ways to ask for keys
--- Copyright © 2014-2016  Clint Adams
+-- Copyright © 2014-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 
 module Codec.Encryption.OpenPGP.KeySelection (
@@ -12,9 +11,6 @@ module Codec.Encryption.OpenPGP.KeySelection (
 ) where
 
 import Codec.Encryption.OpenPGP.Types
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>), (*>))
-#endif
 import Control.Applicative (optional)
 import Control.Monad ((<=<))
 import Crypto.Number.Serialize (i2osp)
