@@ -36,7 +36,7 @@ import Data.Text (Text)
 import Codec.Encryption.OpenPGP.Types
 import Codec.Encryption.OpenPGP.Ontology (isTrustPkt)
 import Data.Conduit.OpenPGP.Keyring.Instances ()
-import Text.ParserCombinators.Incremental.LeftBiasedLocal (concatMany, completeResults, feed, feedEof, inspect, Parser, satisfy)
+import Text.ParserCombinators.Incremental.LeftBiasedLocal (concatMany, completeResults, failure, feed, feedEof, inspect, Parser, satisfy)
 
 parseAChunk :: (Monoid s, Show s) => Parser s r -> s -> ([(r, s)], Maybe (Maybe (r -> r), Parser s r)) -> (([(r, s)], Maybe (Maybe (r -> r), Parser s r)), [r])
 parseAChunk _ a ([], Nothing) = error $ "Failure before " ++ show a
@@ -105,8 +105,10 @@ brokenTK 5 = do
 brokenTK _ = fail "Unexpected broken packet type"
 
 pkPayload :: Parser [Pkt] (PKPayload, Maybe SKAddendum)
-pkPayload = do [PublicKeyPkt p] <- satisfy isPKP
-               return (p, Nothing)
+pkPayload = do pkpkts <- satisfy isPKP
+               case pkpkts of
+                   [PublicKeyPkt p] -> return (p, Nothing)
+                   _ -> failure
     where
         isPKP [PublicKeyPkt _] = True
         isPKP _ = False
@@ -114,8 +116,11 @@ pkPayload = do [PublicKeyPkt p] <- satisfy isPKP
 signature :: Bool -> [SigType] -> Parser [Pkt] [SignaturePayload]
 signature intolerant rts = if intolerant then signature' else signature' <|> brokensig'
     where
-        signature' = do [SignaturePkt sp] <- satisfy (isSP intolerant)
-                        return $! (if intolerant then id else filter isSP') [sp]
+        signature' = do
+          spks <- satisfy (isSP intolerant)
+          case spks of
+              [SignaturePkt sp] -> return $! (if intolerant then id else filter isSP') [sp]
+              _ -> failure
         brokensig' = const [] <$> broken 2
         isSP True [SignaturePkt sp@SigV3{}] = isSP' sp
         isSP True [SignaturePkt sp@SigV4{}] = isSP' sp
@@ -126,25 +131,34 @@ signature intolerant rts = if intolerant then signature' else signature' <|> bro
         isSP' _ = False
 
 signedUID :: Bool -> Parser [Pkt] (UidOrUat, [SignaturePayload])
-signedUID intolerant = do [UserIdPkt u] <- satisfy isUID
-                          sigs <- concatMany (signature intolerant [GenericCert, PersonaCert, CasualCert, PositiveCert, CertRevocationSig])
-                          return (I u, sigs)
+signedUID intolerant = do upkts <- satisfy isUID
+                          case upkts of
+                              [UserIdPkt u] -> do
+                                  sigs <- concatMany (signature intolerant [GenericCert, PersonaCert, CasualCert, PositiveCert, CertRevocationSig])
+                                  return (I u, sigs)
+                              _ -> failure
     where
         isUID [UserIdPkt _] = True
         isUID _ = False
 
 signedUAt :: Bool -> Parser [Pkt] (UidOrUat, [SignaturePayload])
-signedUAt intolerant = do [UserAttributePkt us] <- satisfy isUAt
-                          sigs <- concatMany (signature intolerant [GenericCert, PersonaCert, CasualCert, PositiveCert, CertRevocationSig])
-                          return (A us, sigs)
+signedUAt intolerant = do uapkts <- satisfy isUAt
+                          case uapkts of
+                              [UserAttributePkt us] -> do
+                                  sigs <- concatMany (signature intolerant [GenericCert, PersonaCert, CasualCert, PositiveCert, CertRevocationSig])
+                                  return (A us, sigs)
+                              _ -> failure
     where
         isUAt [UserAttributePkt _] = True
         isUAt _ = False
 
 signedOrRevokedPubSubkey :: Bool -> Parser [Pkt] [(Pkt, [SignaturePayload])]
-signedOrRevokedPubSubkey intolerant = do [p] <- satisfy isPSKP
-                                         sigs <- concatMany (signature intolerant [SubkeyBindingSig, SubkeyRevocationSig])
-                                         return [(p, sigs)]
+signedOrRevokedPubSubkey intolerant = do pskpkts <- satisfy isPSKP
+                                         case pskpkts of
+                                             [p] -> do
+                                                 sigs <- concatMany (signature intolerant [SubkeyBindingSig, SubkeyRevocationSig])
+                                                 return [(p, sigs)]
+                                             _ -> failure
     where
         isPSKP [PublicSubkeyPkt _] = True
         isPSKP _ = False
@@ -155,9 +169,12 @@ brokenPubSubkey = do _ <- broken 14
                      return []
 
 rawOrSignedOrRevokedSecSubkey :: Bool -> Parser [Pkt] [(Pkt, [SignaturePayload])]
-rawOrSignedOrRevokedSecSubkey intolerant = do [p] <- satisfy isSSKP
-                                              sigs <- concatMany (signature intolerant [SubkeyBindingSig, SubkeyRevocationSig])
-                                              return [(p, sigs)]
+rawOrSignedOrRevokedSecSubkey intolerant = do sskpkts <- satisfy isSSKP
+                                              case sskpkts of
+                                                  [p] -> do
+                                                      sigs <- concatMany (signature intolerant [SubkeyBindingSig, SubkeyRevocationSig])
+                                                      return [(p, sigs)]
+                                                  _ -> failure
     where
         isSSKP [SecretSubkeyPkt _ _] = True
         isSSKP _ = False
@@ -168,15 +185,19 @@ brokenSecSubkey = do _ <- broken 7
                      return []
 
 skPayload :: Parser [Pkt] (PKPayload, Maybe SKAddendum)
-skPayload = do [SecretKeyPkt p ska] <- satisfy isSKP
-               return (p, Just ska)
+skPayload = do spkts <- satisfy isSKP
+               case spkts of
+                   [SecretKeyPkt p ska] -> return (p, Just ska)
+                   _ -> failure
     where
         isSKP [SecretKeyPkt _ _] = True
         isSKP _ = False
 
 broken :: Int -> Parser [Pkt] Pkt
-broken t = do [bp] <- satisfy isBroken
-              return bp
+broken t = do bpkts <- satisfy isBroken
+              case bpkts of
+                  [bp] -> return bp
+                  _ -> failure
     where
         isBroken [BrokenPacketPkt _ a _] = t == fromIntegral a
         isBroken _ = False
