@@ -40,7 +40,7 @@ import Data.Word (Word8, Word32)
 import Data.Maybe (fromMaybe)
 import Network.URI (nullURI, parseURI, uriToString)
 
-import Codec.Encryption.OpenPGP.Internal (curveoidBSToCurve, curveToCurveoidBS, pubkeyToMPIs, multiplicativeInverse)
+import Codec.Encryption.OpenPGP.Internal (curveoidBSToCurve, curveoidBSToEdSigningCurve, curveToCurveoidBS, edSigningCurveToCurveoidBS, pubkeyToMPIs, multiplicativeInverse, curve2Curve, curveFromCurve)
 import Codec.Encryption.OpenPGP.Types
 
 instance Binary SigSubPacket where
@@ -264,6 +264,10 @@ getSigSubPacket = do
             | pt == 32 = do
                        sp <- get :: Get SignaturePayload
                        return $ SigSubPacket crit (EmbeddedSignature sp)
+            | pt == 33 = do
+                       kv <- getWord8
+                       fp <- getLazyByteString (if kv == 4 then 20 else 32)
+                       return $ SigSubPacket crit (IssuerFingerprint kv (TwentyOctetFingerprint fp))
             | pt > 99 && pt < 111 = do
                        payload <- getLazyByteString (l - 1)
                        return $ SigSubPacket crit (UserDefinedSigSub pt payload)
@@ -382,6 +386,12 @@ putSigSubPacket (SigSubPacket crit (EmbeddedSignature sp)) = do
     putSubPacketLength . fromIntegral $ (1 + BL.length spb)
     putSigSubPacketType crit 32
     putLazyByteString spb
+putSigSubPacket (SigSubPacket crit (IssuerFingerprint kv fp)) = do
+    let fpb = unTOF fp
+    putSubPacketLength . fromIntegral $ (2 + BL.length fpb)
+    putSigSubPacketType crit 33
+    putWord8 kv
+    putLazyByteString fpb
 putSigSubPacket (SigSubPacket crit (UserDefinedSigSub ptype payload)) = putSigSubPacket (SigSubPacket crit (OtherSigSub ptype payload))
 putSigSubPacket (SigSubPacket crit (OtherSigSub ptype payload)) = do
     putSubPacketLength . fromIntegral $ (1 + BL.length payload)
@@ -786,18 +796,26 @@ getPubkey ECDSA = do
     curvelength <- getWord8 -- FIXME: test for 0 or 0xFF as they are reserved
     curveoid <- getByteString (fromIntegral curvelength)
     MPI mpi <- getMPI -- FIXME: check length against curve type?
-    case curveoidBSToCurve curveoid >>= \c -> mpi2point c mpi of
+    case curveoidBSToCurve curveoid of
         Left e -> fail e
-        Right (curve, point) -> return . ECDSAPubKey . ECDSA_PublicKey . ECDSA.PublicKey curve $ point
-    where
-        mpi2point c cpi = fmap (\y -> (c,y)) (bs2Point (i2osp cpi))
+        Right Curve25519 -> return $ EdDSAPubKey Ed25519 (EPoint mpi)
+        Right curve -> case bs2Point (i2osp mpi) of
+                           Left e -> fail e
+                           Right point -> return . ECDSAPubKey . ECDSA_PublicKey . ECDSA.PublicKey (curve2Curve curve) $ point
 getPubkey ECDH = do
-    ECDSAPubKey ed <- getPubkey ECDSA
+    ed <- getPubkey ECDSA -- could be an ECDSA or an EdDSA
     kdflen <- getWord8 -- FIXME: should be 3, test for 0 or 0xFF as they are reserved
     one <- getWord8 -- FIXME: should be 1
     kdfHA <- get
     kdfSA <- get
     return $ ECDHPubKey ed kdfHA kdfSA
+getPubkey EdDSA = do
+    curvelength <- getWord8 -- FIXME: test for 0 or 0xFF as they are reserved
+    curveoid <- getByteString (fromIntegral curvelength)
+    MPI mpi <- getMPI -- FIXME: check length against curve type?
+    case curveoidBSToEdSigningCurve curveoid of
+        Left e -> fail e
+        Right Ed25519 -> return . EdDSAPubKey Ed25519 $ EPoint mpi
 
 getPubkey _ = UnknownPKey <$> getRemainingLazyByteString
 
@@ -810,8 +828,12 @@ bs2Point bs = let xy = B.drop 1 bs in
 
 putPubkey :: PKey -> Put
 putPubkey (UnknownPKey bs) = putLazyByteString bs
-putPubkey p@(ECDSAPubKey (ECDSA_PublicKey (ECDSA.PublicKey curve _))) = let Right curveoidbs = curveToCurveoidBS curve in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) -- FIXME: do not output length 0 or 0xff
-putPubkey p@(ECDHPubKey (ECDSA_PublicKey (ECDSA.PublicKey curve _)) kha ksa) = let Right curveoidbs = curveToCurveoidBS curve in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) >> putWord8 0x03 >> putWord8 0x01 >> put kha >> put ksa -- FIXME: do not output length 0 or 0xff
+putPubkey p@(ECDSAPubKey (ECDSA_PublicKey (ECDSA.PublicKey curve _))) = let Right curveoidbs = curveToCurveoidBS (curveFromCurve curve) in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) -- FIXME: do not output length 0 or 0xff
+putPubkey p@(ECDHPubKey (ECDSAPubKey (ECDSA_PublicKey (ECDSA.PublicKey curve _))) kha ksa) = let Right curveoidbs = curveToCurveoidBS (curveFromCurve curve) in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) >> putWord8 0x03 >> putWord8 0x01 >> put kha >> put ksa -- FIXME: do not output length 0 or 0xff
+putPubkey p@(ECDHPubKey (EdDSAPubKey curve _) kha ksa) = let Right curveoidbs = curveToCurveoidBS (ed2ec curve) in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) >> putWord8 0x03 >> putWord8 0x01 >> put kha >> put ksa -- FIXME: do not output length 0 or 0xff
+    where
+        ed2ec Ed25519 = Curve25519
+putPubkey p@(EdDSAPubKey curve _) = let Right curveoidbs = edSigningCurveToCurveoidBS curve in putWord8 (fromIntegral (B.length curveoidbs)) >> putByteString curveoidbs >> mapM_ put (pubkeyToMPIs p) -- FIXME: do not output length 0 or 0xff
 putPubkey p = mapM_ put (pubkeyToMPIs p)
 
 getSecretKey :: PKPayload -> Get SKey
