@@ -16,18 +16,23 @@ import Control.Error.Util (hush)
 import Control.Lens ((^.), _1)
 import Control.Monad (liftM2)
 
+import Crypto.Error (eitherCryptoError)
+import Crypto.Hash (hashWith)
 import qualified Crypto.Hash.Algorithms as CHA
 import Crypto.Number.Serialize (i2osp, os2ip)
 import qualified Crypto.PubKey.DSA as DSA
 import qualified Crypto.PubKey.ECC.ECDSA as ECDSA
+import qualified Crypto.PubKey.Ed25519 as Ed25519
 import qualified Crypto.PubKey.RSA.PKCS15 as P15
 import qualified Crypto.PubKey.RSA.Types as RSATypes
 
 import Data.Bifunctor (first)
+import qualified Data.ByteArray as BA
 import qualified Data.ByteString as B
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString.Lazy as BL
 import Data.Either (isRight, lefts, rights)
+import Data.Function (on)
 import Data.IxSet.Typed ((@=))
 import qualified Data.IxSet.Typed as IxSet
 import Data.List.NonEmpty (NonEmpty(..))
@@ -130,12 +135,20 @@ verifyAgainstKeys ks sig mt payload = do
         verify' _ _ _ _ = error "This should never happen (verify')."
         verify'' (DSA,mpis) hd pub (DSAPubKey (DSA_PublicKey pkey)) bs = dsaVerify pub mpis hd pkey bs
         verify'' (ECDSA,mpis) hd pub (ECDSAPubKey (ECDSA_PublicKey pkey)) bs = ecdsaVerify pub mpis hd pkey bs
+        verify'' (EdDSA,mpis) hd pub (EdDSAPubKey Ed25519 pkey) bs = ed25519Verify pub mpis hd (i2osp (unEPoint pkey)) bs
         verify'' (RSA,mpis) hd pub (RSAPubKey (RSA_PublicKey pkey)) bs = rsaVerify pub mpis hd pkey bs
         verify'' _ _ _ _ _ = Left "unimplemented key type"
         dsaVerify pub (r:|[s]) hd pkey bs = if DSA.verify hd pkey (dsaMPIsToSig r s) bs then Right pub else Left ("DSA verification failed: " ++ show (hd,pkey,r,s,bs))
         dsaVerify _ _ _ _ _ = Left "cannot verify DSA signature of wrong shape"
         ecdsaVerify pub (r:|[s]) hd pkey bs = if ECDSA.verify hd pkey (ecdsaMPIsToSig r s) bs then Right pub else Left ("ECDSA verification failed: " ++ show (hd,pkey,r,s,bs))
         ecdsaVerify _ _ _ _ _ = Left "cannot verify ECDSA signature of wrong shape"
+        ed25519Verify pub (r:|[s]) hd pkey bs = either (Left . (("Ed25519 verification failed: " ++ show (hd,pkey,r,s,bs) ++ ": ")++) . show) return $ do
+                                                   ep <- cf2es (Ed25519.publicKey (B.drop 1 pkey)) -- drop the 0x40
+                                                   es <- cf2es (Ed25519.signature ((B.append `on` i2osp . unMPI) r s))
+                                                   let prehash = crazyHash hd bs :: B.ByteString
+                                                   if Ed25519.verify ep prehash es then Right pub else Left ("does not verify")
+        ed25519Verify _ _ _ _ _ = Left "cannot verify Ed25519 signature of wrong shape"
+        cf2es = either (Left . show) return . eitherCryptoError
         rsaVerify pub mpis hd pkey bs = if P15.verify (Just hd) pkey bs (rsaMPItoSig mpis) then Right pub else Left ("DSA verification failed: " ++ show (hd,pkey,mpis,bs))
         dsaMPIsToSig r s = DSA.Signature (unMPI r) (unMPI s)
         ecdsaMPIsToSig r s = ECDSA.Signature (unMPI r) (unMPI s)
@@ -151,6 +164,7 @@ verifyAgainstKeys ks sig mt payload = do
         expiredBefore :: UTCTime -> SigSubPacket -> Bool
         expiredBefore ct (SigSubPacket _ (SigExpirationTime et)) = fromEnum ((posixSecondsToUTCTime . toEnum . fromEnum) et `diffUTCTime` ct) < 0
         expiredBefore _ _ = False
+        crazyHash h = BA.convert . hashWith h
 
 finalPayload :: Pkt -> ByteString -> ByteString
 finalPayload s pl = BL.concat [pl, sigbit, trailer s]
