@@ -245,6 +245,7 @@ data PubKeyAlgorithm = RSA
                      | ECDSA
                      | ForbiddenElgamal
                      | DH
+                     | EdDSA
                      | OtherPKA Word8
     deriving (Show, Data, Generic, Typeable)
 
@@ -264,6 +265,7 @@ instance FutureVal PubKeyAlgorithm where
     fromFVal ECDSA = 19
     fromFVal ForbiddenElgamal = 20
     fromFVal DH = 21
+    fromFVal EdDSA = 22
     fromFVal (OtherPKA o) = o
     toFVal 1 = RSA
     toFVal 2 = DeprecatedRSAEncryptOnly
@@ -274,6 +276,7 @@ instance FutureVal PubKeyAlgorithm where
     toFVal 19 = ECDSA
     toFVal 20 = ForbiddenElgamal
     toFVal 21 = DH
+    toFVal 22 = EdDSA
     toFVal o = OtherPKA o
 
 instance Hashable PubKeyAlgorithm
@@ -288,6 +291,7 @@ instance Pretty PubKeyAlgorithm where
     pretty ECDSA = pretty "ECDSA"
     pretty ForbiddenElgamal = pretty "(forbidden) Elgamal"
     pretty DH = pretty "DH"
+    pretty EdDSA = pretty "EdDSA"
     pretty (OtherPKA pka) = pretty "unknown pubkey algorithm type" <+> pretty pka
 
 $(ATH.deriveJSON ATH.defaultOptions ''PubKeyAlgorithm)
@@ -768,6 +772,7 @@ data SigSubPacketPayload = SigCreationTime ThirtyTwoBitTimeStamp
                   | Features (Set FeatureFlag)
                   | SignatureTarget PubKeyAlgorithm HashAlgorithm SignatureHash
                   | EmbeddedSignature SignaturePayload
+                  | IssuerFingerprint Word8 TwentyOctetFingerprint
                   | UserDefinedSigSub Word8 ByteString
                   | OtherSigSub Word8 ByteString
     deriving (Data, Eq, Generic, Show, Typeable) -- FIXME
@@ -798,6 +803,7 @@ instance Pretty SigSubPacketPayload where
     pretty (Features ffs) = pretty "features" <+> pretty (Set.toList ffs)
     pretty (SignatureTarget pka ha sh) = pretty "signature target" <+> pretty pka <+> pretty ha <+> prettyLBS sh
     pretty (EmbeddedSignature sp) = pretty "embedded signature" <+> pretty sp
+    pretty (IssuerFingerprint kv ifp) = pretty "issuer fingerprint (v" <> pretty kv <> pretty ")" <+> pretty ifp
     pretty (UserDefinedSigSub t bs) = pretty "user-defined signature subpacket type" <+> pretty t <+> pretty (BL.unpack bs)
     pretty (OtherSigSub t bs) = pretty "unknown signature subpacket type" <+> pretty t <+> prettyLBS bs
 
@@ -825,6 +831,7 @@ instance A.ToJSON SigSubPacketPayload where
     toJSON (Features ffs) = object [T.pack "features" .= ffs]
     toJSON (SignatureTarget pka ha sh) = object [T.pack "signatureTarget" .= (pka, ha, BL.unpack sh)]
     toJSON (EmbeddedSignature sp) = object [T.pack "embeddedSignature" .= sp]
+    toJSON (IssuerFingerprint kv ifp) = object [T.pack "issuerFingerprint" .= (kv, ifp)]
     toJSON (UserDefinedSigSub t bs) = object [T.pack "userDefinedSigSub" .= (t, BL.unpack bs)]
     toJSON (OtherSigSub t bs) = object [T.pack "otherSigSub" .= (t, BL.unpack bs)]
 
@@ -1018,11 +1025,18 @@ instance A.ToJSON UserAttrSubPacket where
     toJSON (ImageAttribute ih d) = A.toJSON (ih, BL.unpack d)
     toJSON (OtherUASub t bs) = A.toJSON (t, BL.unpack bs)
 
-data ECCCurve = BrokenNISTP256
-              | BrokenNISTP384
-              | BrokenNISTP521
+data ECCCurve = NISTP256
+              | NISTP384
+              | NISTP521
+              | Curve25519
     deriving (Data, Eq, Generic, Ord, Show, Typeable)
 
+instance Pretty ECCCurve where
+    pretty NISTP256 = pretty "NIST P-256"
+    pretty NISTP384 = pretty "NIST P-384"
+    pretty NISTP521 = pretty "NIST P-521"
+    pretty Curve25519 = pretty "Curve25519"
+
 instance Hashable ECCCurve
 
 newtype Block a = Block {unBlock :: [a]} -- so we can override cereal instance
