@@ -1,5 +1,5 @@
 -- Compression.hs: OpenPGP (RFC4880) compression conduits
--- Copyright © 2012-2014  Clint Adams
+-- Copyright © 2012-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -14,8 +14,8 @@ import Data.Conduit
 import qualified Data.Conduit.List as CL
 import Control.Monad.Trans.Resource (MonadThrow)
 
-conduitCompress :: MonadThrow m => CompressionAlgorithm -> Conduit Pkt m Pkt
+conduitCompress :: MonadThrow m => CompressionAlgorithm -> ConduitT Pkt Pkt m ()
 conduitCompress algo = CL.consume >>= \ps -> yield (compressPkts algo ps)
 
-conduitDecompress :: MonadThrow m => Conduit Pkt m Pkt
+conduitDecompress :: MonadThrow m => ConduitT Pkt Pkt m ()
 conduitDecompress = CL.concatMap decompressPkt
