@@ -1,5 +1,5 @@
 -- Instances.hs: OpenPGP (RFC4880) additional types for transferable keys
--- Copyright © 2012-2015  Clint Adams
+-- Copyright © 2012-2016  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -41,12 +41,12 @@ instance Ord SignaturePayload where
     compare s1@(SigV3 st1 ct1 eoki1 pka1 ha1 left16_1 mpis1) s2@(SigV3 st2 ct2 eoki2 pka2 ha2 left16_2 mpis2) = compare ct1 ct2 <> compare st1 st2 <> compare eoki1 eoki2 -- FIXME: nondeterministic
     compare s1@(SigV4 st1 pka1 ha1 has1 uhas1 left16_1 mpis1) s2@(SigV4 st2 pka2 ha2 has2 uhas2 left16_2 mpis2) = compare (sigCT s1) (sigCT s2) <> compare st1 st2 <> compare (issuer (SignaturePkt s1)) (issuer (SignaturePkt s2)) -- FIXME: nondeterministic
     compare s1@(SigVOther sv1 bs1) s2@(SigVOther sv2 bs2) = compare sv1 sv2 <> compare bs1 bs2
-    compare (SigV3 {}) (SigV4 {}) = LT
-    compare (SigV3 {}) (SigVOther {}) = LT
-    compare (SigV4 {}) (SigV3 {}) = GT
-    compare (SigV4 {}) (SigVOther {}) = LT
-    compare (SigVOther {}) (SigV3 {}) = GT
-    compare (SigVOther {}) (SigV4 {}) = GT
+    compare SigV3{} SigV4{} = LT
+    compare SigV3{} SigVOther{} = LT
+    compare SigV4{} SigV3{} = GT
+    compare SigV4{} SigVOther{} = LT
+    compare SigVOther{} SigV3{} = GT
+    compare SigVOther{} SigV4{} = GT
 
 instance Semigroup TK where
     (<>) a b = TK (_tkKey a)
