@@ -117,7 +117,7 @@ testKeysSelfVerification :: Bool -> FilePath -> Assertion
 testKeysSelfVerification expectsuccess keyfile = do
     ks <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ keyfile) DC..| conduitGet get DC..| conduitToTKs DC..| CL.consume
     let verifieds = mapM (verifyTKWith (verifySigWith (verifyAgainstKeys ks)) Nothing) ks
-    assertEqual (keyfile ++ " self-verification") expectsuccess (isRight verifieds)
+    assertEqual (keyfile ++ " self-verification") expectsuccess (isRight verifieds) -- FIXME:
 
 testKeysExpiration :: Bool -> FilePath -> Assertion
 testKeysExpiration expectsuccess keyfile = do
@@ -281,11 +281,13 @@ unitTests = testGroup "Unit Tests" [
    , testCase "anibal-ed25519.gpg" (testSerialization "anibal-ed25519.gpg")
    , testCase "nist_p-256_key.gpg" (testSerialization "nist_p-256_key.gpg")
    , testCase "nist_p-256_secretkey.gpg" (testSerialization "nist_p-256_secretkey.gpg")
+   , testCase "sample-eddsa.pubkey" (testSerialization "sample-eddsa.pubkey")
    ],
   testGroup "PKA/Size/KeyID/fingerprint group" [
      testCase "v3 key" (testPKAandSizeAndKeyIDandFingerprint "v3.key" "R1024:C7261095/CBD9 F412 6807 E405 CC2D  2712 1DF5 E86E")
    , testCase "v4 key" (testPKAandSizeAndKeyIDandFingerprint "000001-006.public_key" "R1248:D4D54EA16F87040E/421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E")
    , testCase "ECDSA key" (testPKAandSizeAndKeyIDandFingerprint "nist_p-256_key.gpg" "E256:F7708BADD6063224/174C CF12 C571 6D0E 527F  B50E F770 8BAD D606 3224")
+   , testCase "EdDSA key" (testPKAandSizeAndKeyIDandFingerprint "sample-eddsa.pubkey" "w256:8CFDE12197965A9A/C959 BDBA FA32 A2F8 9A15  3B67 8CFD E121 9796 5A9A")
    ],
   testGroup "Keyring group" [
      testCase "pubring 7732CF988A63EA86" (testKeyringLookup "pubring.gpg" "7732CF988A63EA86" True)
@@ -321,11 +323,14 @@ unitTests = testGroup "Unit Tests" [
    ,  testCase "revoked pubkey" (testKeysSelfVerification False "revoked.pubkey")
    ,  testCase "expired pubkey" (testKeysSelfVerification True "expired.pubkey")
    ,  testCase "nist_p-256 pubkey" (testKeysSelfVerification True "nist_p-256_key.gpg")
+   ,  testCase "ed25519 pubkey" (testKeysSelfVerification True "ed25519.pubkey")
    ],
   testGroup "Key expiration group" [
       testCase "6F87040E pubkey" (testKeysExpiration True "6F87040E.pubkey")
    ,  testCase "expired pubkey" (testKeysExpiration False "expired.pubkey")
    ,  testCase "nist_p-256 pubkey" (testKeysExpiration True "nist_p-256_key.gpg")
+   ,  testCase "ed25519-without-curve25519.pubkey" (testKeysExpiration True "ed25519-without-curve25519.pubkey")
+   ,  testCase "ed25519.pubkey" (testKeysExpiration True "ed25519.pubkey")
    ],
   testGroup "Compression group" [
      testCase "compressedsig.gpg" (testCompression "compressedsig.gpg")
